#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/

import socket
import ssl
import tempfile
import sys, os
sys.path.append("..")
sys.path.append(".")
#sys.path.append("../../pyKeySql/keysql_client")
#sys.path.append("../../keysql_client")
sys.path.append("../keysql_client")
import logging
import time
from datetime import datetime
#now = datetime.now().time()    to get time
import threading

import locale
locale.setlocale(locale.LC_ALL, 'en_US.utf-8')
"""
import keysql_client
from keysql_client.keysql_message_header import MessageHeader
from keysql_client.keysql_exception import KeySqlException, KeySqlConnectException, KeySqlExceptionMessage
from keysql_client.keysql_connector_base import KeySqlConnectorBase
"""

#import PyKeySql.keysql_message_header.MessageHeader
#import  keysql_message_header.MessageHeader

#import keysql_message_header

from keysql_client.keysql_message_header import MessageHeader
from keysql_client.keysql_exception import KeySqlException, KeySqlConnectException, KeySqlExceptionMessage
from keysql_client.keysql_connector_base import KeySqlConnectorBase

#from keysql_message_header import MessageHeader

class KeySqlConnector(KeySqlConnectorBase):
    def cut_message(self, mess):
        TOTAL_LEN_BUG_REPORT = 1024 # 320
        if len(mess) > TOTAL_LEN_BUG_REPORT:
            short_mess = mess[:TOTAL_LEN_BUG_REPORT] + ' ... ' + mess[-TOTAL_LEN_BUG_REPORT:] + '__len(mess): {} smbs'.format(len(mess))
        else:
            short_mess = mess
        return short_mess

    def __init__(self, host: str, port:int, schema: str = "", timeout:int = None):
        super(KeySqlConnector, self).__init__(host, port, schema)
        try:
            self._host = socket.gethostbyname(host)
        except ValueError as ex:
            logging.exception("message")
            raise KeySqlConnectException(ex)
        self._port = port
        self._server_address = (self._host, self._port)
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        if timeout:
            self._sock.settimeout(timeout)
        self._sock_connected = False
        self._connected = False
        self._msg_id = 0
        self._sending = False

    def connect_user(self, user, password, schema = None):
        header = None
        try:
            init_msg = None
            if schema:
                init_msg = "connect --user " + user + " --password " + password + " --schema " + schema + "\n"
            else:
                init_msg = "connect " + user + " " + password + "\n"
            self._sock.sendall(bytes(init_msg,encoding="utf-8"))
            header = self.__read_API_call_reply()
        except ValueError as ex:   #   Exception as ex:
            raise KeySqlConnectException(ex)  # .message)

        if header["exit_code"] != 0 or header["err_message"] != "OK":
            #print(reply)
            raise KeySqlConnectException(header["err_message"])
        self._connected = True
        return True

    def __API_call(self,command: str,expected: str):
        header = None
        try:
            while (self._sending):
                time.sleep(0.001)
            self._sending = True
            #logging.info("start_sending1:  " + str(datetime.now().time()))
            self._sock.sendall(bytes(command + "\n",encoding="utf-8"))
            #logging.info("finish_sending1: " + str(datetime.now().time()))
            self._sending = False
            if not expected:
                return True
            header = self.__read_API_call_reply()
        except ValueError as ex:   #   Exception as ex:
            raise KeySqlConnectException(ex)  # .message)

        if header["exit_code"] == 0 and header["err_message"] == expected:
            return True
        raise KeySqlConnectException(header["err_message"])  # .message)
    def __read_API_call_reply(self):
        reply = self._sock.recv(128)
        pos = reply.find(b'\n')
        header = MessageHeader(0, 'keysql').unpack(reply[0:pos].decode("ascii"))
        reply = reply[pos + 1:]
        pos = reply.find(b'\n')
        header["err_message"] = reply[0:pos].decode("ascii")
        return header
    def disconnect_user(self):
        return self.__API_call("disconnect","OK")

    def abort_query(self):
        return self.__API_call("abort","")

    def shutdown(self):
        # TODO: check connections ---> ssl or not .... if ssl - False, for tcp only ---
        if True:
            return self.__API_call("shutdown", "")

    def ping(self):
        return self.__API_call("ping","PONG")

    def reconnect(self,schema : str):
        return self.__API_call("reconnect --schema " + schema,"OK")

    def change_password(self,username : str, old_password :str, new_password : str):
        return self.__API_call("change-password --user " + username + " --old-password " + old_password + " --new-password " + new_password, "OK")

    def _connect(self, user: str = None, password: str = None, schema: str = None):  # , nonblocking=True):
        """
        Connect with server: non-secure connect
        :param user: user name
        :param password: user password
        :return: None
        """

        if self._connected:
            raise KeySqlConnectException(KeySqlExceptionMessage.DISCONNECT_BEFORE_CONNECT)

        if not self._sock_connected:
            try:
                self._sock.connect(self._server_address)

                #self._sock.setblocking(0) # does not work, but doc says it should be
                #if nonblocking:
                #    self._sock.setblocking(0)

                self._sock_connected = True
            except ValueError as ex:  # Exception as ex:
                raise KeySqlConnectException(ex)  # .message)

        if user and password:
            return self.connect_user(user, password, schema)
        return True

    def ssl_connect(self, user: str = None, password: str = None, schema: str = None):
        """
        Connect with server: SSL connection with server
        :param user: user name
        :param password:  user password
        :return: None
        """
        if self._connected:
            raise KeySqlConnectException(KeySqlExceptionMessage.DISCONNECT_BEFORE_CONNECT)

        if not self._sock_connected:
            try:
                context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
                fd, tmp_path = tempfile.mkstemp()
                with open(tmp_path, 'w') as f:
                    for s in self.pem:
                        f.write(s)
                os.close(fd)
                context.load_verify_locations(tmp_path)
                self._sock = context.wrap_socket(self._sock, server_hostname = self._host)
                self._sock.connect(self._server_address)
                #self._sock.setblocking(0)
                #if nonblocking:
                #    self._sock.setblocking(0)
                #os.remove(tmp_path)
                self._sock_connected = True

            except ValueError as ex:
                raise KeySqlConnectException(ex)

        if user and password:
            return self.connect_user(user, password, schema)
        return True


    def send_resv(self, msg, format='keysql'):
        """
        Send a query to the server
        :param msg: str
        :return: None
        """

        def utf8len(s):
            return len(s.encode('utf-8'))

        if format not in ['keysql', 'json', 'packed']:
            raise KeySqlException(-1, KeySqlExceptionMessage.FORMAT_KEYSQL_JSON_PACKED)

        self._msg_id += 1
        header = MessageHeader(self._msg_id, format)
        message = "submit " + header.out_msg(utf8len(msg))
        message += msg

        while (self._sending):
            time.sleep(0.001)
        self._sending = True
        #logging.info("start_sending2:  " + str(datetime.now().time()))
        self._sock.sendall(message.encode())
        #logging.info("finish_sending2: " + str(datetime.now().time()))
        self._sending = False

        reply = ""
        reply_header = None
        header_str = ""
        page = b""

        def get_header(chunk,header_str):
            pos = chunk.find(b'\n')
            if pos > 0:
                header_str += chunk[0:pos].decode("ascii")
                reply_header = header.unpack(header_str)
                return chunk[pos + 1:], "", reply_header

            header_str += chunk.decode("ascii")
            return b"", header_str, None

        while True:
            chunk = self._sock.recv(8192)
            if reply_header is None:
                chunk, header_str, reply_header = get_header(chunk, header_str)
                #if reply_header:
                #   print("=============================", reply_header, len(page),  "=============================xxx")
                if reply_header and reply_header['exit_code'] != 0:
                    page = b""
                    reply = ""
            page += chunk
            if len(page) >= reply_header["message_length"]:
                if len(reply) > 0 and reply[-1] != '\n':
                    reply += '\n'
                reply += page[0:reply_header["message_length"]].decode("utf-8")

                if reply_header["page"] == 0:
                    break
                page, header_str, reply_header = get_header(page[reply_header["message_length"] + 1:],header_str)
                #if reply_header:
                #    print("=============================", reply_header, len(page),  "=============================yyy")

                if reply_header and reply_header["page"] == 0 and len(page) >= reply_header["message_length"]:
                    if reply_header['exit_code'] != 0:
                        reply = page.decode("utf-8")
                    else:
                        reply += page.decode("utf-8")
                    break;

        if reply_header['exit_code'] != 0:
            short_reply = self.cut_message(reply)
            logging.error('exit_code: ' + str(reply_header['exit_code']) + ', reply: ' + short_reply)
            reply_header["err_message"] = short_reply
            raise KeySqlException(reply_header['exit_code'], short_reply)

        return reply_header, reply

    def send_resv_script(self, msg, format='keysql'):
        """
        Send a query to the server
        :param msg: str
        :return: None
        """

        def utf8len(s):
            return len(s.encode('utf-8'))

        if format not in ['keysql', 'json']:
            raise KeySqlException(-1, KeySqlExceptionMessage.FORMAT_KEYSQL_JSON)

        self._msg_id += 1
        header = MessageHeader(self._msg_id, format)
        message = "submit " + header.out_msg(utf8len(msg))
        message += msg

        while (self._sending):
            time.sleep(0.001)
        self._sending = True
        #logging.info("start_sending2:  " + str(datetime.now().time()))
        self._sock.sendall(message.encode())
        #logging.info("finish_sending2: " + str(datetime.now().time()))
        self._sending = False

        reply = ""
        reply_header = None
        header_str = ""
        page = b""

        script_reply = []

        def get_header(chunk,header_str):
            pos = chunk.find(b'\n')
            if pos > 0:
                header_str += chunk[0:pos].decode("ascii")
                reply_header = header.unpack(header_str)
                return chunk[pos + 1:], "", reply_header

            header_str += chunk.decode("ascii")
            return b"", header_str, None

        def check_exit_code(reply_header,reply):
            if reply_header['exit_code'] != 0:
                short_reply = self.cut_message(reply)
                logging.error('exit_code: ' + str(reply_header['exit_code']) + ', reply: ' + short_reply)
                reply_header["err_message"] = short_reply
                raise KeySqlException(reply_header['exit_code'], short_reply)

        while True:
            chunk = self._sock.recv(8192)
            page += chunk
            while True:
                if reply_header is None:
                    page, header_str, reply_header = get_header(page, header_str)
                    if reply_header is None:
                        break
                    if reply_header['exit_code'] != 0:
                        reply = ""
                if len(page) >= reply_header["message_length"]:
                    reply += page[0:reply_header["message_length"]].decode("utf-8")
                    check_exit_code(reply_header,reply)
                    page = page[reply_header["message_length"] + 1:]
                    if reply_header["page"] == 0:
                        script_reply.append((reply_header, reply))
                        if reply_header["query"] == 0:
                            return script_reply
                        reply = ""
                    header_str = ""
                    reply_header = None
                    if len(page) == 0:
                        break
                else:
                    break

        return script_reply

    # do not unpack reply, test only server process query, without unpacking reply
    # return total # of bytes
    def send_skip_resv(self, msg, format='keysql'):
        """
        Send a query to the server
        :param msg: str
        :return: None
        """

        def utf8len(s):
            return len(s.encode('utf-8'))

        if format.lower() not in ['keysql', 'json']:
            raise KeySqlException(-1, KeySqlExceptionMessage.FORMAT_KEYSQL_JSON)

        self._msg_id += 1
        header = MessageHeader(self._msg_id, format)
        message = "submit " + header.out_msg(utf8len(msg))
        message += msg

        while (self._sending):
            time.sleep(0.001)
        self._sending = True
        #logging.info("start_sending2:  " + str(datetime.now().time()))
        self._sock.sendall(message.encode())
        #logging.info("finish_sending2: " + str(datetime.now().time()))
        self._sending = False

        #reply = ""
        reply_header = None
        header_str = ""
        page = b""

        def get_header(chunk,header_str):
            pos = chunk.find(b'\n')
            if pos > 0:
                header_str += chunk[0:pos].decode("ascii")
                reply_header = header.unpack(header_str)
                return chunk[pos + 1:], "", reply_header

            header_str += chunk.decode("ascii")
            return b"", header_str, None

        while True:
            chunk = self._sock.recv(8192)
            if reply_header is None:
                chunk, header_str, reply_header = get_header(chunk, header_str)
                #if reply_header:
                #   print("=============================", reply_header, len(page),  "=============================xxx")
                if reply_header and reply_header['exit_code'] != 0:
                    page = b""
                    #reply = ""
            page += chunk
            if len(page) >= reply_header["message_length"]:
                #reply += page[0:reply_header["message_length"]].decode("utf-8")
                if reply_header["page"] == 0:
                    break
                page, header_str, reply_header = get_header(page[reply_header["message_length"] + 1:],header_str)
                #if reply_header:
                #    print("=============================", reply_header, len(page),  "=============================yyy")

                if reply_header and reply_header["page"] == 0 and len(page) >= reply_header["message_length"]:
                    #if reply_header['exit_code'] != 0:
                    #    reply = page.decode("utf-8")
                    #else:
                    #    reply += page.decode("utf-8")
                    break

        if reply_header['exit_code'] != 0:
            short_reply = "Error from server (when skip reply)"  # self.cut_message(reply)
            logging.error('exit_code: ' + str(reply_header['exit_code']) + ', reply: ' + short_reply)
            reply_header["err_message"] = short_reply
            raise KeySqlException(reply_header['exit_code'], short_reply)

        return reply_header  #  , reply


    def _disconnect(self):
        """
        Disconnect with server
        :return: None
        """
        time.sleep(0.5)     # 5 millisecond - to finish if something still running
        self._sock.close()
        self._connected = False


if __name__ == "__main__":
    pass
