#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/
import logging
from keysql_client.keysql_exception import KeySqlExceptionMessage,KeySqlException
from keysql_client.keysql_statement import KeySqlStatement

class DeleteStatement(KeySqlStatement):
    """
    example1 statement = DeleteStatement().from("billiard_store").where(":num is null")
    example2 statement = DeleteStatement().from("billiard_store").where(":color = {} and :num < {}",'red', my_ball.num)
    """
    _store_name = None
    _where = None

    def __init__(self):
        self._store_name = None
        self._where = None

    def _from(self,store_name):
        self._store_name = store_name
        return self

    def where(self, where_str, args = None):
        if args:
            self._where = where_str.format(args)
        else:
            self._where = where_str
        return self

    def create_request(self):
        if self._store_name is None:
            raise KeySqlException(KeySqlExceptionMessage.STORE_IS_NOT_DEFINED)

        request = "delete from " + self._store_name

        if self.where:
            request += " where " + self._where

        logging.debug('keysql_store_delete request: ' + request)
        return request

    def process_reply(self, reply: str, header = None)-> str:
        # nothin to process - just
        count = header['instance_count'] if header else 0
        return count

if __name__ == "__main__":
    pass