#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/
import logging
from keysql_client.keysql_statement import KeySqlStatement
from keysql_client.keysql_instance import KeySql_Instance

class  InsertIntoStoreInstance(KeySqlStatement):
    """
   Insert Into Store Instance
    """
    _store_name = None
    _instance = None

    def __init__(self, store_name:str = None, instance_or_list:[str, list, KeySql_Instance] = None):
        """Create InsertIntoStoreInstance
        :param store_name (str):
        :param instance_or_list (str, list, KeySql_Instance): string, KeySql_Instance, list of str or KeySql_Instance
        """
        self._store_name = store_name
        self._instance = instance_or_list

    def create_request(self):
        if type(self._instance) == list:
            res = [KeySql_Instance.instances_to_string(item) for item in self._instance]
            #return "[" + ",".join(res) + "]"
            insts = ",".join(res)
        elif type(self._instance) == KeySql_Instance:
            insts = KeySql_Instance.instances_to_string(self._instance)
        else:
            value = None
            if type(self._instance) == str or type(self._instance) == int or type(self._instance) == float:
                value = self._instance
            else:
                value = self._instance.__dict__
            inst = KeySql_Instance(self._instance.__class__.__name__,value)
            insts = KeySql_Instance.instances_to_string(inst)
        request = 'INSERT INTO {} INSTANCES {}'.format(self._store_name, insts)
        logging.debug("update_request" + request)
        return request
        #return 'INSERT INTO {} INSTANCES {}'.format(self._store_name, self._value.serialize())

    def process_reply(self, reply: str, header=None):
        """Return number of inserted instances
        :param reply:
        :param count:
        :return:
        """
        count = header['instance_count'] if header else 0
        return count