"""
#/**********************************************************************************
#* Copyright (C) 2017-2020 by KEYARK, INC. - All Rights Reserved.
#* All information contained herein is proprietary and confidential to KEYARK, INC.
#* Any use, reproduction, or disclosure without the written
#* permission of KEYARK, INC. is prohibited.
#* Written by Natalia Belyaeva <nbelyaeva@keyark.com>
#***********************************************************************************/
"""
import logging
from keysql_client.keysql_mapper import KeySqlMapper
from keysql_client.keysql_statement_type_object import StatementType
from keysql_client.keysql_statement import KeySqlStatement
from keysql_client.keysql_exception import KeySqlException, KeySqlExceptionMessage
import json
from typing import List, Dict

class KeySqlDefaultStatement(KeySqlStatement):
    _to_dict = False # True

    def __init__(self, reply: str = None, to_dict = False, header=None):
        self._reply = reply
        self._to_dict = to_dict
        self._header = header

    def create_request(self):
        return self._reply

    def process_reply(self, reply, header=None, base_classes: list = []) -> None:
        # nothing to process - input is ''
        #return reply
        #print("reply: ", reply, '------------------')
        #print(type(reply))
        logging.debug('reply before process:' + str(reply))

        format = 'keysql' if header is None else header['format']
        # typical header
        # {'message_length': 30, 'id': 3, 'exit_code': 0,
        # 'statement_type': <StatementType.SHOW: 3>, 'instance_count': 0,
        # 'time': 0.001965, 'page': 0,
        # 'format': 'keysql', 'statement_obj': <StatementObj.STORES: 5>, 'err_message': ''}

        if reply == '' or reply == []:
            # return []
            return super().process_reply(reply, header)

        if format == 'json':
            # user example:
            # [{"user":{"user_name":"TEST","must_change_pwd":0,"enabled":1,"default_schema":"TEST","check_expiration":0,
            # "use_ldap":0,"created_at":"2020-11-23 12:21:50.363866","updated_at":"2020-11-23 12:21:50.363866",
            # "pwd_updated_at":"2020-11-23 12:21:50.363866","extended_props":null,"password_policy_name":null}},...]
            if type(reply) is list:
                lead_key = list(reply[0].keys())[0].upper()
                if lead_key == "USER":
                    #return [rec["USER"]["user_name"] for rec in reply]    # before: return user names only
                    return reply
                if lead_key == "USER_NAME":
                    # return [{"USER_NAME":"user_name"} for rec in reply]    # before: return user names only
                    return reply

                # user permission example:
                # [{'USER_PERMISSIONS': {'TYPE': 'user', 'NAME': 'test3', 'GRANTS': [{'GRANT': {'SECURABLE': 'SERVER',
                # 'PERMISSIONS': ['ALL']}}, {'GRANT': {'SECURABLE': 'SCHEMA', 'NAME': 'PUBLIC', 'PERMISSIONS': ['ALL']}},
                # {'GRANT': {'SECURABLE': 'SCHEMA', 'NAME': 'TEST3', 'PERMISSIONS': ['ALL']}}]}}]
                if lead_key == "USER_PERMISSIONS":
                    #return [rec["USER_PERMISSIONS"]["NAME"] for rec in reply] # before: returns user names only
                    return reply
                # sequence example
                # [{'sequence': {'sequence_name': 'MY_SEQUENCE_FULL', 'min_val': 1, 'max_val': 299,
                # 'initial_val': 1, 'increment': 1}}]
                if lead_key == "SEQUENCE":
                    return reply

                # policy example
                # [{'password_policy': {'password_policy_name': 'DAY_1', 'max_age': 2, 'max_age_unit': 'DAYS',
                # 'min_length': 3, 'enforce_history': 0}}]
                if lead_key.upper() == "PASSWORD_POLICY": # "password_policy_name":  # "PASSWORD_POLICY":
                    return reply

            if type(reply) is dict:

                lead_keys = list(reply.keys())
                if len(lead_keys) > 1:

                    # version:  {'name': 'KeySQL Server', 'version': '2.9.0', 'date': '2020-12-01'}
                    if  'name' in lead_keys and 'version' in lead_keys and 'date' in lead_keys:
                        return reply

                    # TODO:
                    print(type(reply))
                    print(reply)
                    print("_____ NOT IMPLEMENTED in keysql statement default flat dict _____")

                lead_key = lead_keys[0]
                if lead_key == "user":
                    return reply #  for now - dictionary ,  reply["user"]["user_name"]
                if lead_key == "USER_PERMISSIONS":
                    return reply # for now - dictionary

            return reply

            # TODO:
            print("==========================================================================")
            print("type(reply): ", type(reply))
            print(reply)
            print("_____ NOT IMPLEMENTED in keysql statement default _____")
            raise Exception('NOT IMPLEMENTED in keysql statement default ------------------- ')
            print("==========================================================================")

            if type(reply) is list:
                return reply

            #TODO: remove later ....
            if type(reply) is str:  # if not json processed ....
                try:
                    tmp = json.loads(reply)
                    return tmp
                except KeySqlException(KeySqlExceptionMessage.WRONG_JSON_FORMAT) as ex:
                    raise KeySqlException(self._reply_header['exit_code'], reply)

        # TODO
        elif format == 'keysql':
            #print(reply)
            #print(type(reply))

            #print('reply default in keysql format1:', reply)
            #reply = reply.replace('\t', '').replace(" ", '').replace('\n', '')
            #reply = reply[1:-1]
            #print('reply default in keysql format2:', reply)

            #instances = KeySqlParser.parse_select_reply(reply)
            instances = reply
            if type(instances) is not list:
                instances = [instances]

            logging.debug('--------->' + str(instances))
            statement_type = StatementType.SELECT if header is None else header['statement_type']
            results = []
            for instance in instances:
                if self._to_dict:
                    curr = KeySqlMapper.keysql_inst_to_dict(instance)
                else:
                    if len(base_classes) == 0:
                        curr = KeySqlMapper.keysql_inst_to_default_class_only(instance)
                    else:
                        curr = KeySqlMapper.keysql_inst_to_value_or_default() if statement_type == StatementType.SHOW else \
                            KeySqlMapper.keysql_inst_to_class_inst(reply, base_classes)

                results.append(curr)

            return results
        else:
            raise KeySqlException(KeySqlExceptionMessage.FORMAT_KEYSQL_JSON)


